-- Core file, event handlers and such.

GravBloxAce = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0")

local GetTime = GetTime
local max = math.max
local pow = math.pow

BloxBoardWidgets = { }
BloxShapeWidgets = { }
BloxNextShapeWidgets = { }
BloxGame = {
   level = 1, 
   score = 0, 
   rows = 0,
}

function BloxGame.numTicksForLevel() 
   return max(1.2*pow(0.9, BloxGame.level), 0.01)
end

function BloxGame:Reset()
   self.level = 1
   self.score = 0
   self.rows = 0
   self.board = BloxBoard:new()
   self.board:RedrawBoard()
   self.board:DrawScore()
end

function GravBloxAce:OnEnable(first)
   --   GravBloxAce:Print("OnEnable")
   local frame_id = 1
   if first then
      BloxFrame:RegisterForDrag("LeftButton")
      BloxFrame.currentTicks = 0   
      BloxFrame.TimeSinceLastUpdate = 0
      BloxGameBoard.WIDTH = 10
      BloxGameBoard.HEIGHT = 23
      
      for x = 1, BloxGameBoard.WIDTH do
	 BloxBoardWidgets[x] = { }
	 for y = 1, BloxGameBoard.HEIGHT do
	    local f = CreateFrame("Frame", nil, BloxGameBoard)
	    f:SetWidth(19)  
	    f:SetHeight(19) 
	    f:SetID(frame_id)
	    f.backdrop = { tile = false }
	    f:SetBackdrop(f.backdrop)
	    frame_id = frame_id + 1
	    f:Show()
	    BloxBoardWidgets[x][y] = f
	 end
      end
      
      for x = 1, 4 do
	 BloxShapeWidgets[x] = { }
	 BloxNextShapeWidgets[x] = { }
	 for y = 1, 4 do
	    -- Frames for the moving shape
	    local f = CreateFrame("Frame", nil, BloxGameBoard)
	    f:SetWidth(19)  
	    f:SetHeight(19)
	    f.backdrop = { tile = false }
	    f:SetBackdrop(f.backdrop)
	    BloxShapeWidgets[x][y] = f

	    -- Frames for the Next shape display
	    f = CreateFrame("Frame", nil, BloxNextPiece)
	    f:SetWidth(18.5)  
	    f:SetHeight(18.5) 
	    f.backdrop = { tile = false }
	    f:SetBackdrop(f.backdrop)
	    BloxNextShapeWidgets[x][y] = f
	 end
      end
      
      BloxGame.board = BloxBoard:new()
      BloxGame.board:RedrawBoard()
   end
end

function GravBloxAce:OnDisable()
   self:CancelScheduledEvent("Blox_OnUpdate")
   BloxGame.board:TogglePaused()
end

function dbg_()
   BloxGame.board.shape:debug()
   BloxGame.board.next:debug()
end

function GravBloxAce.Scheduler()
   local ticks = GetTime()
   BloxFrame.TimeSinceLastUpdate = ticks - BloxFrame.currentTicks
   BloxFrame.currentTicks = ticks
   if BloxGame.board then
      BloxGame.board:Update()
      BloxGame.board:Draw()
   end      
end

function GravBloxAce:OnDragStart()
   BloxFrame:StartMoving()
   self.isMoving = true
end

function GravBloxAce:OnDragStop()
   BloxFrame:StopMovingOrSizing()
   self.isMoving = false
end


function GravBloxAce:OnMouseClick()
   if not self.isGrabbed then
      if BloxGame.board.isGameOver then
	 BloxGame:Reset()
	 BloxGame.board.isPaused = true
      end
      BloxFrame:EnableKeyboard(true)
      self.isGrabbed = true
      BloxGame.board:TogglePaused()
      GravBloxAce:ScheduleRepeatingEvent("Blox_OnUpdate", self.Scheduler, 0.02)
   end
end

function BloxGame:OnKeyDown()
   --GravBloxAce:Print("OnKeyDown: "..arg1)
   if self.isAnimating then return end
   if arg1 == "RIGHT" then 
      self.board.shape:setMove(1)
   elseif arg1 == "LEFT" then 
      self.board.shape:setMove(-1)
   elseif arg1 == "DOWN" then
      self.board.shape:setDropTick( 0.03 )
   elseif arg1 == "UP" then
      self.board.shape:rotate()
   elseif arg1 == "SPACE" then
      self.board.shape:drop()
   end
end

function BloxGame:OnKeyUp()
   --GravBloxAce:Print("OnKeyUp: "..arg1)
   if arg1 == "ESCAPE" then
      BloxFrame:EnableKeyboard(false)
      GravBloxAce.isGrabbed = false
      GravBloxAce:CancelScheduledEvent("Blox_OnUpdate")
      self.board:TogglePaused()
      return
   end

   if arg1 == "RIGHT" or arg1 == "LEFT" then
      self.board.shape:setMove(0)
   elseif arg1 == "DOWN" then
      self.board.shape:setDropTick( BloxGame.numTicksForLevel( ) )
   end
end

      
